;------------------------------------------------------------------------------
; 3DO joypad for AT90S2313  Ver 1.0  2010.1.12
;------------------------------------------------------------------------------

	.include "2313def.inc"

;------------------------------------------------------------------------------

	.def	cnt		= 	r17
	.def	bitcnt	= 	r20
	.def	wrk		= 	r21
	.def	tmp		= 	r22
	.def	rec		=   r23
	.def	pcnt	= 	r18
	
;------------------------------------------------------------------------------

	rjmp	INIT		; Reset Handler
	rjmp	EXT_INT0	; IRQ0 Handler
	rjmp	EXT_INT1	; IRQ0 Handler
	rjmp	TIM1_CAPT	; Timer1 Capture Handler
	rjmp	TIM1_COMP	; Timer1 Compare Handler
	rjmp	TIM1_OVF	; Timer1 Overflow Handler
	rjmp	TIM0_OVF	; Timer0 Overflow Handler
	rjmp	UART_RXC  	; UART RX Complete Handler
	rjmp	UART_DRE	; UDR Empty Handler
	rjmp	UART_TXC	; UART TX Complete Handler
	rjmp	ANA_COMP	; Analog Comparator Handler

;------------------------------------------------------------------------------

EXT_INT0:
EXT_INT1:
TIM1_CAPT:
TIM1_COMP:
TIM1_OVF:
TIM0_OVF:
UART_RXC:
UART_DRE:
UART_TXC:
ANA_COMP:
	reti

; defines interface lines
.equ clkinport = PIND
.equ clkindir = DDRD	
.equ clkinpin = 1
.equ dataoutport = PORTD
.equ dataoutdir = DDRD
.equ dataoutpin = 2
.equ clkoutport = PORTD
.equ clkoutdir = DDRD
.equ clkoutpin = 0
.equ datainport = PIND
.equ dataindir = DDRD
.equ datainpin = 6

;defines key lines
.equ upport = PINB
.equ downport = PINB
.equ leftport = PINB
.equ rightport = PINB
.equ lshiftport = PINB
.equ rshiftport = PINB
.equ xport = PINB
.equ pport = PINB
.equ aport = PIND
.equ bport = PIND
.equ cport = PIND

.equ uppin = 0
.equ downpin = 1
.equ leftpin = 2
.equ rightpin = 3
.equ lshiftpin = 4
.equ rshiftpin = 5
.equ xpin = 6
.equ ppin = 7
.equ apin = 3
.equ bpin = 4
.equ cpin = 5

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

	.org	0x0C

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

Init:
	; **** Initialize Stack Pointer ****
	ldi		tmp, LOW(RAMEND)	; Stack Pointer Setup
	out		SPL, tmp			; Stack Pointer Low Byte	
	; **** Initialize Comparator ****
	ldi		tmp, 0x80			; Disable comparator to save power
	out		ACSR, tmp			; 
	; **** Initialize I/O Port ****
	ldi		tmp, 0x00			; Set PB as Input Mode
	out		DDRB, tmp
	ldi		tmp, 0xff			; Pulup on
	out		PORTB, tmp

	ldi		tmp, 1<<clkoutpin | 1<< dataoutpin	
	out		DDRD, tmp			; Only clkout & dataout pin is output
	ldi		tmp, ~(1<<clkoutpin )			; Pulup on
	out		PORTD, tmp


	; **** Main Loop ****
;------------------------------------------------------------------------------

Main:
	clr		cnt
	clr	    pcnt
	cbi 	clkoutport,clkoutpin   ; clockout  0

SearchRise:
	sbis	clkinport, clkinpin    ;   
	rjmp	End
	sbi		clkoutport,clkoutpin   ; clockout  1
	inc		cnt
	cpi		cnt, 0x20  		 
	brne	SearchRise		       ;   
		
;     
;   	
	clr		tmp
	lsl		tmp
	sbis	downport,downpin
	sbr		tmp,1
	lsl		tmp
	sbis	upport,uppin
	sbr		tmp,1
	lsl		tmp
	sbis	rightport,rightpin
	sbr		tmp,1
	lsl		tmp
	sbis	leftport,leftpin
	sbr		tmp,1
	lsl		tmp
	sbis	aport,apin
	sbr		tmp,1
	ori		tmp, 0x80 
	
	ldi		ZH, 0	
	ldi		ZL, 0x60  
	std		Z+0, tmp
	
	clr		tmp
	lsl		tmp
	sbis	bport,bpin
	sbr		tmp,1
	lsl		tmp
	sbis	cport,cpin
	sbr		tmp,1
	lsl		tmp
	sbis	pport,ppin
	sbr		tmp,1
	lsl		tmp
	sbis	xport,xpin
	sbr		tmp,1
	lsl		tmp
	sbis	rshiftport,rshiftpin
	sbr		tmp,1
	lsl		tmp
	sbis	lshiftport,lshiftpin
	sbr		tmp,1
	lsl		tmp
	lsl		tmp 

	std		Z+1, tmp 
	

SearchFallen:    					;   
	sbic	clkinport, clkinpin
	rjmp	SearchFallen
	cbi 	clkoutport,clkoutpin  	; clockout  0

StartSend:	
	
	cpi		pcnt, 0x08  		 
	breq	Finish
	ldi		ZH, 0	
	ldi		ZL, 0x60     
	rcall	SendBytes
	inc 	pcnt
	std		Z+0, tmp
	std		Z+1, rec
;  tmp    0xff      
	mov 	wrk,tmp
	com		wrk 
	brne	StartSend
Finish:
	sbi		dataoutport, dataoutpin		//    

End:	
	rjmp	Main

;-------------------------------------------

SendBytes:
	clr		rec
	ldi 	cnt,2			;  2 
	ld		wrk, Z+
	ldi		bitcnt, 0x8		;  	

JP_NEXT_STEP0:	
	sbrs	wrk, 7
	cbi		dataoutport, dataoutpin
	sbrc	wrk, 7
	sbi		dataoutport, dataoutpin
	lsl		wrk
	

JP_NEXT_STEP1:		//    
	sbis	clkinport, clkinpin
	rjmp	JP_NEXT_STEP1
	sbi 	clkoutport,clkoutpin  	; clockout  1	
	lsl		rec
	sbic	datainport,datainpin	;     data  
	sbr		rec, 1
	dec		bitcnt
	brne	JP_NEXT_STEP2
	ldi		bitcnt, 0x8
	dec		cnt
	breq	JP_PROCESS_END
	; 8    joy2 
	mov		tmp, rec
	clr 	rec
	ld		wrk, Z+
JP_NEXT_STEP2:		//   
	sbic	clkinport, clkinpin
	rjmp	JP_NEXT_STEP2
	cbi 	clkoutport,clkoutpin  	; clockout  0
	rjmp	JP_NEXT_STEP0
	
JP_PROCESS_END:
	ldi		ZH, 0			;  Z
	ldi		ZL, 0x60 
	ret








